==> asterisk-fastsmsapp-1.03.tar: asterisk_1.0.x/app_fastsms.c <==
/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * FastSMS application - Interface to Bayham Systems SMS Services
 * 
 * Copyright (C) 2005, Mark Sadler, Bayham Systems Limited
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */

#include <asterisk/lock.h>
#include <asterisk/logger.h>
#include <asterisk/options.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>
#include <asterisk/config.h>
#include <asterisk/callerid.h>
#include <asterisk/utils.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <dirent.h>
#include <ctype.h>
#include <curl/curl.h>



static char *app = "FastSMS";

static char *tdesc = "SMS Handler";

static char *synopsis = "Interface to Bayham Systems SMS Services";

static char *descrip = "  FastSMS(msisdn|tezt|[senderid]|[mwi]):  FastSMS allows text messages to be sent via\n"
"the Bayham System SMS Service.\n"
"Two arguments are required, the destination number and the text of the message.\n"
"The destination number must be include the country code at the beginning\n"
"The text message will be truncated if it is more than 160 characters.\n"
"Optionally you can also supply a short string (max 11 chars) to indicate the sender\n"
"this could be a text string, a phone number or a short code.\n"
"You can also supply a flag to set/clear the message waiting indicators on the mobile phone.\n"
"The MWI parameter must have a value in the range 1-8\n";

STANDARD_LOCAL_USER;

/*LOCAL_USER_DECL;*/

CURL *curl;

#define FASTSMS_CONFIG "fastsms.conf"

static char *paramDelimiters = "|";

/* Command processing */

static int fastsms_exec(struct ast_channel *chan, void *data) {
  struct ast_config *cfg;
  char *companyid;
  char *userid;
  char *password;
  char *msisdn;
  char *text;
  char *originator;
  char *mwi;
  char *params;
  long responseCode;

  ast_log(LOG_DEBUG,"FastSMS exec started\n");

  if (!data || ast_strlen_zero((char *)data)) {
    ast_log(LOG_ERROR, "FastSMS requires arguments (msisdn,message text)\n");
    return -1;
  }
      
  /* Load config params */
  cfg = ast_load(FASTSMS_CONFIG);
  if (!cfg) {
    ast_log(LOG_ERROR,"FastSMS configuration file (%s) is missing",FASTSMS_CONFIG);
    return -1;
  }
  companyid = ast_variable_retrieve(cfg,"general","companyid");
  userid = ast_variable_retrieve(cfg,"general","userid");
  password = ast_variable_retrieve(cfg,"general","password");
  if (!companyid) {
    ast_log(LOG_ERROR,"companyid parameter is missing from %s",FASTSMS_CONFIG);
    return -1;
  }
  if (!userid) {
    ast_log(LOG_ERROR,"userid parameter is missing from %s",FASTSMS_CONFIG);
    return -1;
  }
  if (!password) {
    ast_log(LOG_ERROR,"password parameter is missing from %s",FASTSMS_CONFIG);
    return -1;
  }
      
  /* Get params */
  msisdn = strtok(data,paramDelimiters);
  text = strtok(0l,paramDelimiters);
  originator = strtok(0l,paramDelimiters);
  mwi = strtok(0l,paramDelimiters);

  /* If params not okay exit */
  if (!msisdn) {
    ast_log(LOG_ERROR,"FastSMS msisdn parameter missing\n");
    return -1;
  }
  if (!text) {
    ast_log(LOG_ERROR,"FastSMS text parameter missing\n");
    return -1;
  }
  if (mwi) {
    char ch=*mwi;
    switch (ch) {
    case '1':
    case '2':
    case '3':
    case '4':
    case '5':
    case '6':
    case '7':
    case '8':
      break;
    default:
      ast_log(LOG_ERROR,"FastSMS mwi parameter invalid, must be 1-8");
      return -1;
    }
  }
  ast_log(LOG_DEBUG,"FastSMS (%s,%s,%s,%s)\n",msisdn,text,originator,mwi); 

  /* Send mesage */
  if (originator) {
    if (mwi) {
      asprintf(&params,"CompanyId=%s&UserId=%s&Password=%s&Msisdn=%s&MessageText=%s&OasText=%s&Mwi=%s",companyid,userid,password,msisdn,text,originator,mwi);
    } else {
      asprintf(&params,"CompanyId=%s&UserId=%s&Password=%s&Msisdn=%s&MessageText=%s&OasText=%s",companyid,userid,password,msisdn,text,originator);
    }
  } else {
    asprintf(&params,"CompanyId=%s&UserId=%s&Password=%s&Msisdn=%s&MessageText=%s",companyid,userid,password,msisdn,text);
  }
  curl_easy_setopt(curl,CURLOPT_URL,"https://secure.bayhamsystems.com/mfi/sendMessage");
  curl_easy_setopt(curl,CURLOPT_POSTFIELDS,params);
  curl_easy_setopt(curl,CURLOPT_SSL_VERIFYPEER,0);
  curl_easy_perform(curl);
  responseCode = 0;
  curl_easy_getinfo(curl,CURLINFO_HTTP_CODE,&responseCode);
  ast_log(LOG_EVENT,"FastSMS response code was %ld\n",responseCode);
  free(params);
  if (responseCode != 200) {
    ast_log(LOG_ERROR,"FastSMS exec failed (%ld)\n",responseCode);
    return -1;
  } else {
    ast_log(LOG_DEBUG,"FastSMS exec complete\n");
    return 0;
  }
}




/* Asterisk Module Interface */

//! Initialize the module
/*!
 * This function is called at module load time.  Put all code in here
 * that needs to set up your module's hardware, software, registrations,
 * etc.
 */
int load_module(void) {
  curl = curl_easy_init();
  return ast_register_application (app, fastsms_exec, synopsis, descrip);
} 

//! Cleanup all module structures, sockets, etc
/*!
 * This is called at exit.  Any registrations and memory allocations need
 * to be unregistered and free'd here.  Nothing else will do these for you (until exit).
 * Return 0 on success, or other than 0 if there is a problem.
 */
int unload_module(void) {
  curl_easy_cleanup(curl);
  return ast_unregister_application (app);
}

//! Provides a usecount
/*!
 * This function will be called by various parts of asterisk.  Basically, all it has
 * to do is to return a usecount when called.  You will need to maintain your usecount
 * within the module somewhere.
 */
int usecount(void) {
  return 0;
}

//! Description
/*!
 * Returns a short description of your module.
 */
char *description(void) {
  return tdesc;
}

//! Returns the ASTERISK_GPL_KEY
/*!
 * This returns the ASTERISK_GPL_KEY, signifiying that you agree to the terms of
 * the GPL stated in the ASTERISK_GPL_KEY.  Your module will not load if it does
 * not return the EXACT message, i.e.  char *key(void){return ASTERISK_GPL_KEY;}
 */
char *key(void) {
  return ASTERISK_GPL_KEY;
}

//! Reload stuff
/*!
 * This function is where any reload routines take place.  Re-read config files,
 * change signalling, whatever is appropriate on a reload.
 * Return 0 on success, and other than 0 on problem.
 */
int reload(void) {
  return 0;
}

==> asterisk-fastsmsapp-1.04.tar: asterisk_1.4.x/app_fastsms.c <==
/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * FastSMS application - Interface to Bayham Systems SMS Services
 * 
 * Copyright (C) 2005, Mark Sadler, Bayham Systems Limited
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */

#include <asterisk/lock.h>
#include <asterisk/logger.h>
#include <asterisk/options.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>
#include <asterisk/config.h>
#include <asterisk/callerid.h>
#include <asterisk/utils.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <dirent.h>
#include <ctype.h>
#include <curl/curl.h>



static char *app = "FastSMS";

static char *synopsis = "Interface to Bayham Systems SMS Services";

static char *descrip = "  FastSMS(msisdn|tezt|[senderid]|[mwi]):  FastSMS allows text messages to be sent via\n"
"the Bayham System SMS Service.\n"
"Two arguments are required, the destination number and the text of the message.\n"
"The destination number must be include the country code at the beginning\n"
"The text message will be truncated if it is more than 160 characters.\n"
"Optionally you can also supply a short string (max 11 chars) to indicate the sender\n"
"this could be a text string, a phone number or a short code.\n"
"You can also supply a flag to set/clear the message waiting indicators on the mobile phone.\n"
"The MWI parameter must have a value in the range 1-8\n";

CURL *curl;

#define FASTSMS_CONFIG "fastsms.conf"

static char *paramDelimiters = "|";

/* Command processing */

static int fastsms_exec(struct ast_channel *chan, void *data) {
  struct ast_config *cfg;
  const char *companyid;
  const char *userid;
  const char *password;
  char *msisdn;
  char *text;
  char *originator;
  char *mwi;
  char *params;
  long responseCode;

  ast_log(LOG_DEBUG,"FastSMS exec started\n");

  if (!data || ast_strlen_zero((char *)data)) {
    ast_log(LOG_ERROR, "FastSMS requires arguments (msisdn,message text)\n");
    return -1;
  }
      
  /* Load config params */
  cfg = ast_config_load(FASTSMS_CONFIG);
  if (!cfg) {
    ast_log(LOG_ERROR,"FastSMS configuration file (%s) is missing",FASTSMS_CONFIG);
    return -1;
  }
  companyid = ast_variable_retrieve(cfg,"general","companyid");
  userid = ast_variable_retrieve(cfg,"general","userid");
  password = ast_variable_retrieve(cfg,"general","password");
  if (!companyid) {
    ast_log(LOG_ERROR,"companyid parameter is missing from %s",FASTSMS_CONFIG);
    return -1;
  }
  if (!userid) {
    ast_log(LOG_ERROR,"userid parameter is missing from %s",FASTSMS_CONFIG);
    return -1;
  }
  if (!password) {
    ast_log(LOG_ERROR,"password parameter is missing from %s",FASTSMS_CONFIG);
    return -1;
  }
      
  /* Get params */
  msisdn = strtok(data,paramDelimiters);
  text = strtok(0l,paramDelimiters);
  originator = strtok(0l,paramDelimiters);
  mwi = strtok(0l,paramDelimiters);

  /* If params not okay exit */
  if (!msisdn) {
    ast_log(LOG_ERROR,"FastSMS msisdn parameter missing\n");
    return -1;
  }
  if (!text) {
    ast_log(LOG_ERROR,"FastSMS text parameter missing\n");
    return -1;
  }
  if (mwi) {
    char ch=*mwi;
    switch (ch) {
    case '1':
    case '2':
    case '3':
    case '4':
    case '5':
    case '6':
    case '7':
    case '8':
      break;
    default:
      ast_log(LOG_ERROR,"FastSMS mwi parameter invalid, must be 1-8");
      return -1;
    }
  }
  ast_log(LOG_DEBUG,"FastSMS (%s,%s,%s,%s)\n",msisdn,text,originator,mwi); 

  /* Send mesage */
  if (originator) {
    if (mwi) {
      asprintf(&params,"CompanyId=%s&UserId=%s&Password=%s&Msisdn=%s&MessageText=%s&OasText=%s&Mwi=%s",companyid,userid,password,msisdn,text,originator,mwi);
    } else {
      asprintf(&params,"CompanyId=%s&UserId=%s&Password=%s&Msisdn=%s&MessageText=%s&OasText=%s",companyid,userid,password,msisdn,text,originator);
    }
  } else {
    asprintf(&params,"CompanyId=%s&UserId=%s&Password=%s&Msisdn=%s&MessageText=%s",companyid,userid,password,msisdn,text);
  }
  curl_easy_setopt(curl,CURLOPT_URL,"https://secure.bayhamsystems.com/mfi/sendMessage");
  curl_easy_setopt(curl,CURLOPT_POSTFIELDS,params);
  curl_easy_setopt(curl,CURLOPT_SSL_VERIFYPEER,0);
  curl_easy_perform(curl);
  responseCode = 0;
  curl_easy_getinfo(curl,CURLINFO_HTTP_CODE,&responseCode);
  ast_log(LOG_EVENT,"FastSMS response code was %ld\n",responseCode);
  free(params);
  if (responseCode != 200) {
    ast_log(LOG_ERROR,"FastSMS exec failed (%ld)\n",responseCode);
    return -1;
  } else {
    ast_log(LOG_DEBUG,"FastSMS exec complete\n");
    return 0;
  }
}




/* Asterisk Module Interface */

//! Initialize the module
/*!
 * This function is called at module load time.  Put all code in here
 * that needs to set up your module's hardware, software, registrations,
 * etc.
 */
static int load_module(void) {
  curl = curl_easy_init();
  return ast_register_application (app, fastsms_exec, synopsis, descrip);
} 

//! Cleanup all module structures, sockets, etc
/*!
 * This is called at exit.  Any registrations and memory allocations need
 * to be unregistered and free'd here.  Nothing else will do these for you (until exit).
 * Return 0 on success, or other than 0 if there is a problem.
 */
static int unload_module(void) {
  curl_easy_cleanup(curl);
  return ast_unregister_application (app);
}


AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY,"FastSMS SMS Sender");




//! Provides a usecount
/*!
 * This function will be called by various parts of asterisk.  Basically, all it has
 * to do is to return a usecount when called.  You will need to maintain your usecount
 * within the module somewhere.
 */
/*int usecount(void) {
  return 0;
  }*/

//! Description
/*!
 * Returns a short description of your module.
 */
/*char *description(void) {
  return tdesc;
  }*/

//! Returns the ASTERISK_GPL_KEY
/*!
 * This returns the ASTERISK_GPL_KEY, signifiying that you agree to the terms of
 * the GPL stated in the ASTERISK_GPL_KEY.  Your module will not load if it does
 * not return the EXACT message, i.e.  char *key(void){return ASTERISK_GPL_KEY;}
 */
/*char *key(void) {
  return ASTERISK_GPL_KEY;
  }*/

//! Reload stuff
/*!
 * This function is where any reload routines take place.  Re-read config files,
 * change signalling, whatever is appropriate on a reload.
 * Return 0 on success, and other than 0 on problem.
 */
static int reload(void) {
  return 0;
  }
